# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""
import math
from datetime import datetime
from pathlib import Path
import time
import importlib_resources

import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pandas as pd
import scipy as sp

pvlib_imported = False
try:
    import pvlib
    pvlib_imported = True
except ImportError:
    print("pvlib import failed")

from hsr1 import DBDriver

from hsr1.plots import (
    TimeDayGraph,
    ElvAziGraph,
    ClearnessDensityGraph,
    LinearTimeGraph,
    LinearDipsGraph,
    DailyDipsSummary,
    LatLonGraph,
    DailyPlots,
    DailyHists,
    SpectrumGraph,
    flagData)

import hsr1.plots.graphUtils as graphUtils
from hsr1.utils.reformatData import ReformatData as reformat
import hsr1.utils.HSRFunc as HsrFunc



class Graph:
    def __init__(self, 
                 data=None, 
                 driver:DBDriver=None, 
                 dataframe=None, 
                 deployment_metadata:pd.DataFrame=None, 
                 accessory_data:pd.DataFrame=None, 
                 output_location:str=None, 
                 diffuse_name:str="DIF", 
                 timezone:str="+00:00",
                 dpi:int=300,
                 **kwargs):
        """a general class for easy access to all the different graph options
        params:
            data: to be used as a positional argument, as either a DBDriver or 
                    a dataframe containing all the data(these can also be passed as keyword arguments),
                    or a string of the database name
            driver: to pass a db_driver object as a keyword argument
            dataframe: to pass a dataframe of data as a keywork argument
            deployment_metadata: to pass a dataframe of deployment metadata as a keyword argurment
            accessory_data: to pass a dataframe of accessory_data as a keyword argurment
            output_location: String filepath to a folder where all the graphs generated will be stored
            diffuse_name: what to call the diffuse on graph labels
            timezone: the desired timezone of the data, if loading from a database
            dpi: dots per inch, pixel resolution of the data
            **kwargs: keyword argurments to pass to load()
        
        this contains many examples of how the underlying graph classes in hsr1.plots
        can be used. there are combined plots with several similar graphs,
        some functions that provide a simple wrapper around a class, and some that
        load data, do some data processing, and then use one of the plots from hsr1.plots
        """
        self.driver = driver
        self.dataframe = dataframe
        self.deployment_metadata = deployment_metadata
        self.accessory_data = accessory_data
        self.diffuse_name = diffuse_name
        self.timezone, self.timedelta = reformat().calculate_timezone(timezone)
        self.output_location = output_location
        self.kwargs = kwargs
        
        if isinstance(data, pd.DataFrame):
            self.dataframe = data
        elif isinstance(data, DBDriver):
            self.driver = data
        elif isinstance(data, str):
            print("db_name was passed")
            self.driver = DBDriver(data)
        
        if output_location is not None:
            Path(output_location).mkdir(parents=True, exist_ok=True)
        
        if self.driver is None and self.dataframe is None and self.accessory_data is None:
            raise ValueError("graph must have data or a database driver")
        
        if self.dataframe is None:
            self.dataframe = pd.DataFrame()
            
        self.jet_black_zero = self.__make_colormap()
        
        matplotlib.rcParams["figure.dpi"] = dpi
        
        SMALL_SIZE = 8
        MEDIUM_SIZE = 10
        BIGGER_SIZE = 15
        
        matplotlib.rc('font', size=MEDIUM_SIZE)
        matplotlib.rc('axes', titlesize=BIGGER_SIZE)
        matplotlib.rc('axes', labelsize=MEDIUM_SIZE)
        matplotlib.rc('xtick', labelsize=SMALL_SIZE)
        matplotlib.rc('ytick', labelsize=SMALL_SIZE)
        matplotlib.rc('legend', fontsize=SMALL_SIZE)
        matplotlib.rc('figure', titlesize=BIGGER_SIZE)
    
    
    
    
    """--------------------
    line plots and presets:
    -------------------"""
    def plot_daily_line(self, 
                        columns:[str], period="monthly", 
                        rows:int=None, days_in_row:int=None, 
                        flag:bool=False, 
                        ignore_zero:bool=False,
                        title_prefix="", dataframe=None, 
                        max_limit=None, min_limit=None):
        """loads some columns from the database and plots them as daily line graphs
        params:
            columns: list of column headers to plot
            period(int or string): how many days to plot per page, options: "weekly", "monthly", int
            rows: number of rows per page
            days_in_row: number of days per row
            flag: whether or not to superimpose flagged data onto the graphs
            ignore_zero: whether or not to plot zeros
            title_prefix: string that will display before the title
        """
        data_columns = columns.copy()
        
        timezone = self.timezone
        ##### calculate the dni if requested
        dni=False
        if "direct_normal_integral" in data_columns:
            dni=True
            data_columns += ["sza", "global_integral", "diffuse_integral"]
            ##### can't load the dni, as it's not in the database
            data_columns.remove("direct_normal_integral")
        
        ##### add the necessary columns for flagging
        if flag and np.isin(["global_integral", "diffuse_integral"], data_columns).any():
            data_columns += ["toa_hi", "sza"]
        
        ##### one liner to remove duplicates from a list while preserving the order
        data_columns = list(dict.fromkeys(data_columns).keys())
        
        data = None
        if dataframe is None:
            data = self.dataframe.copy()
        else:
            data = dataframe.copy()
        
        if np.logical_not(np.isin(data_columns, data.columns)).any():
            print("loading data from the database")
            data = self.driver.db_load.load(data_columns+["pc_time_end_measurement"], timezone=timezone, **self.kwargs)
        
        if dni:
            data["direct_normal_integral"] = ((data["global_integral"]-data["diffuse_integral"])/np.cos(data["sza"]))
        
        if ignore_zero:
            data = data.replace({0:np.nan})
        
        ##### uses the passed value of deployment metadata as a default, 
        deployment_metadata = self.deployment_metadata
        if deployment_metadata is None and self.driver is not None:
            ##### loads everything from the deployment metadata and generates the title and description
            deployment_metadata = self.driver.db_load.load_metadata()
        
        if deployment_metadata is None:
            print("No Deployment metadata available for the title")
        else:
            title_prefix = deployment_metadata["deployment_description"][0] + "\n" + title_prefix
        
        dp = DailyPlots(columns, data, title_prefix, 
                        output_location=self.output_location, flag=flag, 
                        max_limit=max_limit, min_limit=min_limit)
        dp.plot_series(period, rows, days_in_row, data)
        
        plt.show()
    
    def daily_integrals(self, period="monthly", rows=None, days_in_row=None, flag=True):
        """daily plot preset of dni, ghi, and diffuse"""
        columns = ["direct_normal_integral", "global_integral", "diffuse_integral"]
        self.plot_daily_line(columns, period, rows, days_in_row, flag, "sunlight intensity in ")
    
    def daily_temps(self, period="monthly", rows=None, days_in_row=None, flag=True):
        """daily plot preset of all different temperature measurements"""
        columns = ["T_CPU", "T_Bezel", "T_RH", "T_Baro"]
        self.plot_daily_line(columns, period, rows, days_in_row, flag, True, "temperatures in ")
    
    def daily_aod_cimel(self, aod_type:str="aod_microtops", wavelengths:list=None, 
                        period=21, rows=3, days_in_row=7, 
                        upper_limit=2, lower_limit=0,
                        clearsky_filter:str="wood", clearsky_filter_kwargs:dict={}):
        """daily plot of the aod at the wavelengths measured by cimel spectrometers
        these wavelengths are: [380, 440, 500, 675, 870, 1020]
        
        params:
            aod_type: one of: total_od, aod_microtops, aod_wood_2017
                which aod calculation to use.
            period(int or string): how many days to plot per page, options: "weekly", "monthly", int
            rows: number of rows per page.
            days_in_row: number of days per row.
            upper_limit: the max value to display.
            lower_limit: the minimum value to display.
            clearsky_filter: which clearsky filter algorithm to use.
            clearsky_filter_kwargs: keyword arguments to pass to clearsky_filter.
        """
        cimel_wavelengths=False
        if wavelengths is None:
            cimel_wavelengths=True
            wavelengths = np.array([380, 440, 500, 675, 870, 1020])
        else:
            wavelengths = np.array(wavelengths)
        
        requirements = ["pc_time_end_measurement", "global_spectrum", "diffuse_spectrum", "sza", "sed", "global_integral", "diffuse_integral"]
        data = self.driver.db_load.load(requirements, raise_on_missing=False, timezone=self.timezone, **self.kwargs)
        data = data.drop_duplicates(["pc_time_end_measurement"])
        
        deployment_metadata = None
        if self.deployment_metadata is None and self.driver is not None:
            deployment_metadata = self.driver.load_metadata()
        else:
            deployment_metadata = self.deployment_metadata
        
        if deployment_metadata is not None:
            title = deployment_metadata["deployment_description"][0] + "\n"
        
        aod_data = HsrFunc.calc_aod_from_df(data, cimel_wavelengths)
        aod_data[["global_integral", "diffuse_integral", "sza"]] = data[["global_integral", "diffuse_integral", "sza"]]
        
        aod = np.stack(aod_data[aod_type].values)[:, wavelengths-300] if not cimel_wavelengths else aod_data[aod_type].values
        
        clearsky_filter = HsrFunc.calculate_clearsky_filter(data, method=clearsky_filter, kwargs=clearsky_filter_kwargs)
        aod = aod[clearsky_filter]
        time_col = aod_data["pc_time_end_measurement"][clearsky_filter].reset_index(drop=True)
        
        limited_df = pd.DataFrame(list(aod), columns=wavelengths.astype(str))
        limited_df["pc_time_end_measurement"] = time_col
        
        self.plot_daily_line(wavelengths.astype(str), 
                             period, rows, days_in_row, 
                             dataframe=limited_df, 
                             max_limit=upper_limit, min_limit=0, 
                             title_prefix=aod_type+"\n")
        
        
    """-------------------
    hist plots and presets
    -------------------"""
    
    def plot_daily_hist(self, columns:[str], title:[str]="", **kwargs):
        """plots a histogram of each column, one column per day
        params:
            columns: list of column headers that will be plotted
                    OR string of one column name
            title: the title of the plot
        """
        if type(columns) == str:
            columns = [columns]
        data = self.dataframe.copy()
        if np.logical_not(np.isin(columns, data.columns)).any():
            if self.driver is not None:
                print("loading data from the database")
                data = self.driver.db_load.load(columns+["pc_time_end_measurement"], **self.kwargs)
        else:
            raise ValueError("db_driver was not passed and the requested columns are not in the dataframe")
        dh = DailyHists(data)
        dh.plot_hists(columns, title, **kwargs)
    
    def voltage_hists(self):
        """daily histogram preset of all voltage measurements"""
        columns = ["_15Vin", "_3VCAP", "Vcc", "VSpare"]
        data = self.dataframe.copy()
        if np.logical_not(np.isin(columns, data.columns)).any():
            print("loading data from the database")
            data = self.driver.db_load.load(columns+["pc_time_end_measurement"])
        data[columns] /= 1000
        dh = DailyHists(data)
        dh.plot_hists(columns, "Voltages", ignore_zero=True)
        
    def pht_hists(self):
        """daily histogram preset of pressure, humidity and temperature"""
        columns = ["pressure", "rh", "baro_temp"]
        data = self.dataframe.copy()
        if np.logical_not(np.isin(columns, data.columns)).any():
            print("loading data from the database")
            data = self.driver.db_load.load(columns+["pc_time_end_measurement"])
        dh = DailyHists(data)
        dh.plot_hists(columns, "Pressure, humidity and temperature", ignore_zero=True)
    
    def current_hists(self):
        """daily histogram preset of all current measurements"""
        columns = ["I_Tot", "I_15VCAM", "I_15VPC", "ISpare"]
        data = self.dataframe.copy()
        if np.logical_not(np.isin(columns, data.columns)).any():
            print("loading data from the database")
            data = self.driver.db_load.load(columns+["pc_time_end_measurement"])
        dh = DailyHists(data)
        dh.plot_hists(columns, "Currents", ignore_zero=False)
    
    
    
    
    def biggest_dips(self, n=15, cutoff_angle=np.radians(80), cutoff_wavelength=1000,
                      date_format:str="%y-%m-%d %H:%M",
                      title=None):
        """whole dataset plot of the n biggest peaks for each measurement, looks like several horizontal lines
        params:
            n: the number of peaks that will be selected per measurement
            cutoff_angle = the maximum zenith angle included, used for excluding nighttime
            cutoff_wavelength: the maximum wavelength included
        """
        global_spectrum = self.dataframe.copy()
        if np.logical_not(np.isin(["pc_time_end_measurement", "global_spectrum"], global_spectrum.columns)).any():
            global_spectrum = self.load_spectrum(cutoff_angle, **self.kwargs)
        
        linearDipsGraph = LinearDipsGraph()
        linearDipsGraph.plot_n_biggest_dips(global_spectrum, n, cutoff_wavelength)
    
    def daily_biggest_dips(self, n:int=15, cutoff_angle:float=np.radians(80), 
                            cutoff_wavelength:int=1000,
                            date_format:str="%H:%M"):
        """one plot per day of the the n biggest peaks for each measurement
        params:
            n: the number of peaks that will be selected per measurement
            cutoff_angle = the maximum zenith angle included, used for excluding nighttime
            cutoff_wavelength: the maximum wavelength included
        """
        global_spectrum = self.dataframe.copy()
        if np.logical_not(np.isin(["pc_time_end_measurement", "global_spectrum"], global_spectrum.columns)).any():
            global_spectrum = self.load_spectrum(cutoff_angle)
        
        linearDipsGraph = LinearDipsGraph()
        linearDipsGraph.plot_biggest_dips_day(global_spectrum, n, cutoff_wavelength, date_format)
    
    
    def plot_time_day(self, column:str, stack_resolution="max", max_integral=2):
        """plots a column from the database on a time/day plot
        params:
            column: the column to plot
            stack_resolution: how to resolve conflicts where there are more readings than pixels
                options="max", "mean", "min"
            max_integral: the max value that the colourmap extends over
        """
        data = self.dataframe.copy()
        if np.logical_not(np.isin([column], data.columns)).any():
            print("loading data from the database")
            data = self.driver.db_load.load([column]+["pc_time_end_measurement"], timezone=self.timezone, **self.kwargs)
        
        fig = plt.figure(figsize=(11.7, 8.3))
        axes = fig.subplots(1)
        td = TimeDayGraph(self.jet_black_zero, max_integral)
        td.fig = fig
        td.time_day_graph(data, axes, "pc_time_end_measurement", column, 
                          stack_resolution=stack_resolution)
        axes.set_ylabel(column)
        
    
    
    def plot_elv_azi(self, column:str):
        """plots a graph of elevation against azimuth, with the colour representing
        the largest of a given value measured with each pair of elevation and azimuth values.
        
        currently data from accessory_datadoes not work, only data from 
            spectral_data, system_data, and precalculated_values works.
        
        params:
            column: the column that will be represented by the colourmap
        """
        # TODO: make accessory_data work
        # truncate dosent work, assumes 1min data
        # use code from reformat_data? 
        
        data = self.dataframe.copy()
        if np.logical_not(np.isin([column], data.columns)).any():
            print("loading data from the database")
            data = self.driver.db_load.load([column]+["pc_time_end_measurement", "sza", "azimuth"], timezone=self.timezone, **self.kwargs)
        
        fig = plt.figure(figsize=(11.7, 8.3))
        axes = fig.subplots(1)
        elv_azi = ElvAziGraph()
        elv_azi.fig = fig
        elv_azi.elv_azi_graph(axes, data, column)
        plt.show()
    
    
    
    
    
    def plot_aod_day(self, clearsky_filter:str="wood", clearsky_filter_kwargs:dict={}):
        """Plots the Aerosol Optical Depth across the spectrum, one page per day
        params:
            clearsky_filter: which clearsky filtering method to use. currently only "wood" is implemented.
                if None, no filtering is applied
            clearsky_filter_kwargs: keyword arguments to pass to the clearsky_filter method,
                these will vary depending on which filtering method is used
        
        """
        requirements = ["pc_time_end_measurement", "global_spectrum", "diffuse_spectrum", "sza", "sed", "global_integral", "diffuse_integral"]
        data = self.driver.db_load.load(requirements, raise_on_missing=False, timezone=self.timezone, **self.kwargs)
        data = data.drop_duplicates(["pc_time_end_measurement"])
        
        deployment_metadata = None
        if self.deployment_metadata is None and self.driver is not None:
            deployment_metadata = self.driver.load_metadata()
        else:
            deployment_metadata = self.deployment_metadata
        
        spectrometer_average_period = 60
        if deployment_metadata is not None:
            title = deployment_metadata["deployment_description"][0] + "\n"
            spectrometer_average_period = float(deployment_metadata["spectrometer_average_period"][0])
        
        spec_day = SpectrumGraph(cmap=self.jet_black_zero, timedelta=self.timedelta, spectrometer_average_period=spectrometer_average_period)
        
        ##### calculate aod values
        aod_data = HsrFunc.calc_aod_from_df(data)
        aod_data[["global_integral", "diffuse_integral"]] = data[["global_integral", "diffuse_integral"]]
        
        clearsky_filter = HsrFunc.calculate_clearsky_filter(data, method=clearsky_filter, kwargs=clearsky_filter_kwargs)
        
        all_daytime_timestamps = pd.to_datetime(data.loc[data["sza"] < np.radians(90), "pc_time_end_measurement"]).dt.tz_localize(None)
        
        all_daytime_times = all_daytime_timestamps.dt.time
        first_time = np.min(all_daytime_times)
        last_time = np.max(all_daytime_times)
        
        spec_day.first_time, spec_day.last_time = first_time, last_time
        
        all_dates = all_daytime_timestamps.dt.date
        days = np.unique(all_dates)
        for day in days:
            day_data = aod_data[aod_data["pc_time_end_measurement"].dt.date == day]
            
            day_clearsky_filter = clearsky_filter[aod_data["pc_time_end_measurement"].dt.date == day]
            
            if len(day_data[day_clearsky_filter]) == 0:
                continue
            
            fig = plt.figure(figsize=(11.7, 8.3))
            
            this_title = title + "Spectral AOD data on "+day.strftime("%Y-%m-%d")
            fig.suptitle(this_title)
            
            axes = fig.subplots(4)
            
            spec_day.fig = fig
            spec_day.plot_all_aod(day_data[day_clearsky_filter].reset_index(drop=True), axes, 2)
            
            spec_day.supplementary_integral_plot(day_data, axes, first_time, last_time, day, spec_day)
            
            pc_time_secs = (data["pc_time_end_measurement"].dt.tz_localize(None).astype("int64")/10**9).values.astype(int)
            aod_500 = np.stack(aod_data["total_od"].values)[:, 500]*500
            axes[3].plot(pc_time_secs, aod_500, label="optical depth x500")
            axes[3].legend()
            
            if self.output_location is not None:
                full_title = graphUtils.make_full_title(this_title)
                plt.savefig(self.output_location+"/aod_day "+str(day)+".png")
            
            plt.show()
    
    def plot_spectrum_day(self, normalisation=None):
        """plots each spectrum over a day on a colour intensity plot
        params:
            normalisation: how to normalise the data default None
                None: no normalisation
                toa_integral: divided by the top of atmosphere integral
                pvlib: normalises against a spectrum generated by pvlib
        """
        spec_day = SpectrumGraph(cmap=self.jet_black_zero, timedelta=self.timedelta)
        
        requirements = []
        data = None
        if len(self.dataframe) == 0:
            requirements = ["gps_latitude", "gps_longitude", "gps_altitude"]
            requirements += spec_day.requirements["all"].copy()
            requirements += ["global_integral", "diffuse_integral"]
            
            data = self.driver.db_load.load(requirements, raise_on_missing=False, timezone=self.timezone, **self.kwargs)
            data = data.drop_duplicates(["pc_time_end_measurement"])
        else:
            data = self.dataframe
        
        
        
        deployment_metadata = None
        if self.deployment_metadata is None and self.driver is not None:
            deployment_metadata = self.driver.load_metadata()
        else:
            deployment_metadata = self.deployment_metadata
        
        if deployment_metadata is not None:
            title = deployment_metadata["deployment_description"][0] + "\n"
            spec_day.spectrometer_average_period = int(deployment_metadata["spectrometer_average_period"][0])
        
        data["direct_normal_spectrum"] = HsrFunc.calc_direct_normal_spectrum(data["global_spectrum"], 
                                                                             data["diffuse_spectrum"], 
                                                                             data["sza"])
        data["toa_ni"] = data["toa_hi"]/np.cos(data["sza"])
        
        if normalisation is not None and normalisation=="toa_integral":
            max_toa = np.max(data["toa_hi"])
            data["global_spectrum"] = data["global_spectrum"]/(data["toa_hi"])#/max_toa)
            data["diffuse_spectrum"] = data["diffuse_spectrum"]/(data["toa_hi"])#/max_toa)
            data["direct_normal_spectrum"] = data["direct_normal_spectrum"]/data["toa_ni"]
        elif normalisation is not None and normalisation=="pvlib":
            if not pvlib_imported:
                raise ImportError("pvlib was not successfully imported")
            pvlib_data = data.copy()
            pvlib_data["sza_deg"] = np.degrees(data["sza"])
            pvlib_data["azimuth_deg"] = np.degrees(data["azimuth"])
            pvlib_data["dayofyear"] = data["pc_time_end_measurement"].dt.day_of_year
            ref_spectra = pvlib.spectrum.spectrl2(pvlib_data["sza_deg"].values, #zenith
                                                  pvlib_data["sza_deg"].values, #aoi
                                                  0, #surface_tilt
                                                  0, #ground_albedo
                                                  101325, #surface_pressure
                                                  pvlib.atmosphere.get_relative_airmass(pvlib_data["sza_deg"].values),
                                                  0.5, #precipitable_water
                                                  0.3, #atmospheric_ozone
                                                  0.1, #aerosol_turbidity
                                                  pvlib_data["dayofyear"].values)
            ref_wavelength = [ref_spectra["wavelength"]]
            
            def interp_and_divide(measured_spectra, reference_spectra):
                reference_spectra = sp.interpolate.interpn(ref_wavelength, reference_spectra, np.arange(300, 1101, 1)).T
                normalised_spectra = np.stack(measured_spectra)/reference_spectra
                normalised_spectra = np.nan_to_num(normalised_spectra, nan=0.0)
                normalised_spectra = np.array(np.split(normalised_spectra, len(normalised_spectra)))
                normalised_spectra = list(normalised_spectra[:, 0, :])
                
                return normalised_spectra
            
            data["global_spectrum"] = interp_and_divide(data["global_spectrum"].values, ref_spectra["poa_global"])
            data["diffuse_spectrum"] = interp_and_divide(data["diffuse_spectrum"].values, ref_spectra["dhi"])
            data["direct_normal_spectrum"] = interp_and_divide(data["direct_normal_spectrum"].values, ref_spectra["dni"])
            
        ##### find the max reading from the global and diffuse spectra
        #####  dni is not included as this gets very high at sunrise/sunset
        all_readings = np.array((np.stack(data.loc[data["sza"]<np.radians(80), "global_spectrum"].values), 
                                 np.stack(data.loc[data["sza"]<np.radians(80), "diffuse_spectrum"].values)))
        all_readings[np.logical_or(all_readings==np.inf, all_readings==-np.inf)] = np.nan
        all_readings = all_readings[:, :, :720]
        max_reading = np.nanmax(all_readings)
        
        all_daytime_timestamps = pd.to_datetime(data.loc[data["sza"] < np.radians(90), "pc_time_end_measurement"]).dt.tz_localize(None)
        all_daytime_times = all_daytime_timestamps.dt.time
        first_time = np.min(all_daytime_times)
        last_time = np.max(all_daytime_times)
        spec_day.first_time, spec_day.last_time = first_time, last_time
        
        all_dates = all_daytime_timestamps.dt.date
        days = np.unique(all_dates)
        for day in days:
            day_data = data[data["pc_time_end_measurement"].dt.date == day]
            
            fig = plt.figure(figsize=(11.7, 8.3))
            axes = fig.subplots(4)
            
            spec_day.fig = fig
            spec_day.plot_all(day_data, axes)
            
            this_title = title + "Spectral data on "+day.strftime("%Y-%m-%d")
            fig.suptitle(this_title)
            
            spec_day.supplementary_integral_plot(day_data, axes, first_time, last_time, day, spec_day, True)
            
            plt.tight_layout()
            
            if self.output_location is not None:
                full_title = graphUtils.make_full_title(this_title)
                plt.savefig(self.output_location+"/spectrum_day_"+str(day)+".png")
            plt.show()
    
    
    def plot_dips_summary(self, n:int=15, cutoff_angle:float=np.radians(80), 
                          cutoff_wavelength:float=1000, 
                          reference_lines:list=None, 
                          reference_labels:list=None,
                          title=""):
        """histogram of the largest dips over a dataset
        params:
            n: the number of peaks that will be selected per measurement
            cutoff_angle: the maximum zenith angle included, used for excluding nighttime
            cutoff_wavelength: the maximum wavelength included
            reference_lines: list of wavelengths where dip should appear, 
                there is a default with the most prominent dips, 
                but you can provide your own if you want
            reference_labels: list of labels for the passed reference lines.
                must be the the same length as reference_lines, use ""
                if you dont want to label them all
        """
        print("plotting dips summary")
        global_spectrum = self.dataframe.copy()
        if np.logical_not(np.isin(["pc_time_end_measurement", "global_spectrum"], global_spectrum.columns)).any():
            global_spectrum = self.load_spectrum(cutoff_angle, **self.kwargs)
        
        dailyDipsSummary = DailyDipsSummary(self.timezone, reference_lines, reference_labels)
        fig = dailyDipsSummary.plot_daily_peaks_summary(global_spectrum, n, cutoff_wavelength)
        
        deployment_metadata = None
        if self.deployment_metadata is None and self.driver is not None:
            deployment_metadata = self.driver.load_metadata()
        else:
            deployment_metadata = self.deployment_metadata
        
        if title == "" and deployment_metadata is not None:
            title = deployment_metadata["deployment_description"][0]
        
        fig.suptitle(title+"\n", x=0.47)
        
        if deployment_metadata is not None:
            metadata_string = graphUtils.generate_metadata_string(deployment_metadata, global_spectrum)
            plt.figtext(x=0.47, y=0.93, s=metadata_string, horizontalalignment="center")
        
        print("dips summary plotted")
        
        if self.output_location is not None:
            plt.savefig(self.output_location+"/dips_summary.png")
        
    
    
    def plot_integral(self, flag=False, title="", max_integral=None):
        """plots a summary of the integral data in a dataset"""
        
        print("plotting summary")
        
        
        time_day_graph = TimeDayGraph(self.jet_black_zero, self.timezone, diffuse_name=self.diffuse_name)
        elv_azimuth_graph = ElvAziGraph(diffuse_name=self.diffuse_name)
        clearness_graph = ClearnessDensityGraph(diffuse_name=self.diffuse_name)
        linear_graph = LinearTimeGraph(self.timezone, diffuse_name=self.diffuse_name)
        
        graphs = [time_day_graph, elv_azimuth_graph, clearness_graph, linear_graph]
        
        time_day_graph.timezone = self.timezone
        
        requirements = []
        data = None
        if len(self.dataframe) == 0:
            requirements = ["gps_latitude", "gps_longitude", "gps_altitude"]
            for x in graphs:
                new_requirements = x.requirements["all"].copy()
                
                ##### this value will be calculated from other values, so is not in the database
                not_in_db_reqs = ["direct_normal_integral"]
                for not_in_db_req in not_in_db_reqs:
                    if not_in_db_req in new_requirements:
                        new_requirements.remove(not_in_db_req)
                requirements += new_requirements
            
            data = self.driver.db_load.load(requirements, raise_on_missing=False, timezone=self.timezone, **self.kwargs)
        
            data = data.drop_duplicates(["pc_time_end_measurement"])
        
        else:
            data = self.dataframe
        
        if "gps_longitude" in requirements and not "gps_longitude" in data.columns:
            deployment_metadata = self.driver.db_load.load_metadata(["default_longitude", "default_latitude", "default_elevation"]).iloc[0]
            data["gps_longitude"] = deployment_metadata["default_longitude"]
            data["gps_latitude"] = deployment_metadata["default_latitude"]
            data["gps_altitude"] = deployment_metadata["default_elevation"]
        
        
        
        ##### filling in columns that arent in the database
        data["direct_normal_integral"] = ((data["global_integral"]-data["diffuse_integral"])/np.cos(data["sza"]))
        
        deployment_metadata = None
        if self.deployment_metadata is None and self.driver is not None:
            deployment_metadata = self.driver.db_load.load_metadata()
        else:
            deployment_metadata = self.deployment_metadata
        
        if max_integral is None:
            ##### calculating the maximum integral, will be shared across all graphs
            max_global_integral = max(data["global_integral"])
            max_normal_integral = np.max(data.loc[np.degrees(data["sza"]) < 80, "direct_normal_integral"])
            max_integral = max(max_global_integral, max_normal_integral)
            
            # max_integral = max(data["global_integral"])
            
            ##### adding 10% padding to the top
            max_integral = (int((max_integral+10)/100)+1)*100
                           
        for graph in graphs:
            graph.max_integral = max_integral
        
        flags = None
        if flag:
            flags = flagData.flag(data, ignore_nights=True)
        
        
        
        ##### make the subplot that the graphs will be plotted onto
        layout = [["ghi_time_day",  "ghi_elv_azi",  "ghi_elv_azi"],
                  ["diff_time_day", "diff_elv_azi", "diff_elv_azi"],
                  ["dni_time_day",  "dni_elv_azi",  "dni_elv_azi"],
                  ["ghi_linear",    "ghi_toa",      "sza_diff_ghi"],
                  ["diff_linear",   "diff_toa",     "ghi_dni_clearness"],
                  ["dni_linear",    "dni_toa",      "ghi_diff_clearness"]]
        
        
        ##### if the height and weight ratios are off, add 1s to the end to stop error
        gridspec_kw = {"height_ratios":(1,), "width_ratios":(3, 1, 1)}
        if len(gridspec_kw["height_ratios"]) < len(layout):
            gridspec_kw["height_ratios"] = tuple(list(gridspec_kw["height_ratios"]) + [1]* (len(layout) - len(gridspec_kw["height_ratios"])))
        
        if len(gridspec_kw["width_ratios"]) < len(layout[0]):
            gridspec_kw["width_ratios"] = tuple(list(gridspec_kw["width_ratios"]) + [1]* (len(layout[0]) - len(gridspec_kw["width_ratios"])))
        
        
        fig = plt.figure("integral", layout="constrained", figsize=(11.7, 8.3))
        axes = fig.subplot_mosaic(layout, gridspec_kw=gridspec_kw, )
        
        if title == "" and deployment_metadata is not None:
            title = deployment_metadata["deployment_description"][0]
        fig.suptitle(title+"\n")
        
        if deployment_metadata is not None:
            metadata_string = graphUtils.generate_metadata_string(deployment_metadata, data)
            plt.figtext(x=0.5, y=0.93, s=metadata_string, horizontalalignment="center")
        
        try:
            integral_axes = [axes["ghi_time_day"], axes["diff_time_day"], axes["dni_time_day"]]
            time_day_graph.graph_all(integral_axes, data[time_day_graph.requirements["all"]], fig, show_cbar=False, flags=flags, timedelta=self.timedelta)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to graph time/day summary\n"+str(e))
        
        
        try:
            linear_integral_axes = [axes["ghi_linear"], axes["diff_linear"], axes["dni_linear"]]
            linear_graph.graph_all(linear_integral_axes, data[linear_graph.requirements["all"]], flags=flags)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to graph linear integral summary\n"+str(e))
        
        try:
            elv_azimuth_axes = [axes["ghi_elv_azi"], axes["diff_elv_azi"], axes["dni_elv_azi"]]
            elv_azimuth_graph.graph_all(elv_azimuth_axes, data[elv_azimuth_graph.requirements["all"]], fig, flags=flags, show_horizon=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to graph elevation/azimuth summary\n"+str(e))
        
        flags = flagData.flag(data, ignore_nights=True)
        
        try:
            clearness_axes = [axes["ghi_toa"], axes["diff_toa"], axes["dni_toa"], axes["sza_diff_ghi"], axes["ghi_dni_clearness"], axes["ghi_diff_clearness"]]
            clearness_graph.graph_all(clearness_axes, data[clearness_graph.requirements["all"]], fig, flags=flags, flag_all=False, match_density=True)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to graph clearness summary\n"+str(e))
        
        
        print("summary plotted")
        
        if self.output_location is not None:
            plt.savefig(self.output_location+"/integral_summary.png")
        plt.show()
        
        
        
    
    def plot_accessory(self, title=""):
        """generates a page of plots summarising the data in accessory_data
        contains:
            pressure, humidity and a comparison of the different temperature measurements
            voltage summary
            current summary
        """
        
        print("plotting accessory_data")
        
        spectral_data = None
        data = None
        if self.accessory_data is None:
            ##### all the data that will be loaded from the accessory table of the database
            requirements = ["pc_time_end_measurement", 
                            "StatusFlags",
                            "RH", 
                            "Pressure", 
                            "T_Bezel", 
                            "T_Baro", 
                            "_15Vin", 
                            "Vcc", 
                            "I_Tot", 
                            "I_15VPC", 
                            "I_15VCAM",
                            "Latitude",
                            "Longitude"]
            
            table_headers = self.driver.db_load.load_table_names()
            
            if not "accessory_data" in table_headers.keys():
                print("Failed to plot accessory, this database has no accessory_data table")
            
            data = self.driver.db_load.load_accessory(requirements, timezone=self.timezone, **self.kwargs)
            data = data.loc[data["StatusFlags"] != 0.0]
        
        else:
            data = self.accessory_data
        
        has_spectral_data = False
        if len(self.dataframe) == 0 and self.driver is not None:
            ##### any data that will be loaded from the spectral table
            #####   they have different lengths so they have to be loaded seperately
            spectral_requirements = ["pc_time_end_measurement",
                                     "camera_temp"]
            spectral_data = self.driver.db_load.load(spectral_requirements, **self.kwargs)
            has_spectral_data = True
            
        elif len(self.dataframe) == 0:
            spectral_data = self.dataframe
            has_spectral_data = True
        
        
        layout = [["rh", "15vin"],
                  ["pressure", "vcc"],
                  ["tbezel", "itot"],
                  ["tbaro", "i15v"],
                  ["tcam", "icam"]]
        
        fig = plt.figure("integral", layout="constrained", figsize=(11.7, 8.3))
        axes = fig.subplot_mosaic(layout)
        
        
        dh = DailyHists(data.copy(), fig)
        
        spectral_dh = None
        if has_spectral_data:
            spectral_dh = DailyHists(spectral_data, fig)
        
        try:
            dh.plot_one_hist("RH", axes["rh"], ignore_zero=True, show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot relative humidity\n"+str(e))
        try:
            dh.plot_one_hist("Pressure", axes["pressure"], ignore_zero=True, show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot presure\n"+str(e))
        
        ##### temps should have identical y-axes, so they are easier to compare,
        #####   this selects the largest and smallest values from any temp sensor to use as limits
        non_zero_temps = data.loc[np.logical_and((data["T_Baro"] != 0.0), (data["T_Bezel"] != 0.0))]
        accessory_min = 0
        accessory_max = 0
        if len(non_zero_temps) != 0:
            accessory_min = np.min(non_zero_temps[["T_Baro", "T_Bezel"]].to_numpy())
            accessory_max = np.max(non_zero_temps[["T_Baro", "T_Bezel"]].to_numpy())
        
        temp_ymin, temp_ymax = None, None
        
        if has_spectral_data:
            cam_min = min(spectral_data["camera_temp"])
            cam_max = max(spectral_data["camera_temp"])
            temp_ymin = min(accessory_min, cam_min)
            temp_ymax = max(accessory_max, cam_max)
        else:
            temp_ymin = accessory_min
            temp_ymax = accessory_max
        
        ten_percent = abs(temp_ymax-temp_ymin)*0.1
        temp_ymax += ten_percent
        temp_ymin -= ten_percent
        
        ##### plots the temperatures, with the calculated limits
        try:
            dh.plot_one_hist("T_Bezel", axes["tbezel"], ignore_zero=True, ylims=(temp_ymin, temp_ymax), show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot bezel temp\n"+str(e))
        try:
            dh.plot_one_hist("T_Baro", axes["tbaro"], ignore_zero=True, ylims=(temp_ymin, temp_ymax), show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot barometer temp\n"+str(e))
        if has_spectral_data:
            try:
                spectral_dh.plot_one_hist("camera_temp", axes["tcam"], ignore_zero=True, ylims=(temp_ymin, temp_ymax))
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed to plot camera temp\n"+str(e))
        
        ##### plots the voltages
        try:
            dh.plot_one_hist("_15Vin", axes["15vin"], ignore_zero=True, show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot 15Vin\n"+str(e))
        try:
            dh.plot_one_hist("Vcc", axes["vcc"], ignore_zero=True, show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot Vcc\n"+str(e))
        
        ##### calculates the limits for the shared current axes(see comment above temps)
        non_zero_temps = data.loc[np.logical_and((data["I_Tot"] != 0.0), (data["I_15VPC"] != 0.0))]
        current_ymin = np.min(non_zero_temps[["I_Tot", "I_15VPC"]].to_numpy())
        current_ymax = np.max(non_zero_temps[["I_Tot", "I_15VPC"]].to_numpy())*1.1
        
        ##### plots the currents
        try:
            dh.plot_one_hist("I_Tot", axes["itot"], ignore_zero=True, zero_axes=True, ylims=(current_ymin, current_ymax), show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot I_Tot\n"+str(e))
        try:
            dh.plot_one_hist("I_15VPC", axes["i15v"], ignore_zero=True, zero_axes=True, ylims=(current_ymin, current_ymax), show_xticks=False)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot I_15VPC\n"+str(e))
        try:
            dh.plot_one_hist("I_15VCAM", axes["icam"], ignore_zero=False, zero_axes=True)
        except KeyboardInterrupt as e:
            raise e
        except Exception as e:
            print("failed to plot I_15VCAM\n"+str(e))
        
        ##### uses the passed value of deployment metadata as a default, 
        deployment_metadata = self.deployment_metadata
        if deployment_metadata is None and self.driver is not None:
            ##### loads everything from the deployment metadata and generates the title and description
            deployment_metadata = self.driver.db_load.load_metadata()
        
        if deployment_metadata is None:
            print("No Deployment metadata available for the title")
        else:
            if title == "":
                title = deployment_metadata["deployment_description"][0]
            fig.suptitle(title+"\n")
            metadata_string = graphUtils.generate_metadata_string(deployment_metadata, data)
            plt.figtext(x=0.5, y=0.93, s=metadata_string, horizontalalignment="center")
        
        print("accessory_data plotted")
        if self.output_location is not None:
            plt.savefig(self.output_location+"/accessory_summary.png")
        plt.show()
        

    def plot_gps(self, title=""):
        """plots a summary of gps related data
        contains:
            plots of gps position thougout the dataset, with the colour representing the time and reading density
            a plot of the difference between pc time and gps time. log scale with linear segment
            a plot of gps age. log scale with linear segment
            plot of number of satelites in view
            plot of tilt: sqrt(pitch^2*roll^2)
        """
        print("plotting GPS")
        
        ##### data priority: prefer accessory_data over GPS, prefer directly passed data over data from a database
        gps_type = "accessory"
        data = self.accessory_data        
        if data is None:
            tables = None
            if self.driver is not None:
                tables = self.driver.load_table_names().keys()
            if self.driver is not None and "accessory_data" in tables:
                gps_type = "accessory"
                if self.driver is not None:
                    requirements = ["pc_time_end_measurement",
                                    "StatusFlags",
                                    "gps_time", 
                                    "GPSAge",
                                    "NSV",
                                    "Roll", "Pitch",
                                    "Latitude", "Longitude"]
                    data = self.driver.load_accessory(requirements, timezone=self.timezone, **self.kwargs)
            
            ##### use data passed into self.data if it fits
            elif (self.dataframe is not None and 
            "longitude" in self.dataframe.columns and 
            "latitude" in self.dataframe.columns and 
            "pc_time_end_measurement" in self.dataframe.columns):
                gps_type = "GPS"
                data = self.data
            
            elif self.driver is not None and "system_data" in tables:
                gps_type = "GPS"
                
                if self.driver is not None:
                    requirements = ["pc_time_end_measurement",
                                    "gps_longitude",
                                    "gps_latitude",
                                    "gps_time",
                                    "gps_status",
                                    "baro_temp",
                                    "pressure",
                                    "rh"]
                    data = self.driver.db_load.load(requirements)
                    
            
            else:
                raise ValueError("no data for gps")
        
        
        layout = [["gps_density", "time_diff"],
                  ["gps_density", "gps_age"],
                  ["gps_time", "NSV"],
                  ["gps_time", "tilt"]]
        fig = plt.figure("integral", layout="constrained", figsize=(11.7, 8.3))
        axes = fig.subplot_mosaic(layout)
        
        
        
        data = data.copy()
        
        ##### ignore all the data where StatusFlags is 0
        #####   this filters out all the rows where all the data is 0
        if gps_type == "accessory":
            data = data.loc[data["StatusFlags"] != 0.0]
        else:
            data = data.loc[~data["gps_time"].isnull()]
            data = data.loc[data["gps_longitude"] != ""]
            data[["gps_longitude", "gps_latitude"]] = data[["gps_longitude", "gps_latitude"]].astype(float)
        
        
        ##### locations where gps_time is 0
        zero_times = data["gps_time"].str.slice(0, 1) == "0"
        
        gps_times = data["gps_time"].copy()
        gps_times[zero_times] = np.nan
        ##### converts gps_time and pc time to an integer number of seconds
        gps_time_index = pd.DatetimeIndex(gps_times, dtype="datetime64[s]")
        gps_time_int = gps_time_index.to_numpy().astype(float)
        gps_time_int[zero_times] = np.nan
        
        pc_time_index = pd.DatetimeIndex(data["pc_time_end_measurement"].dt.tz_convert(None), dtype="datetime64[s]")
        pc_time_int = pc_time_index.to_numpy().astype(float)
        pc_time_int[zero_times] = np.nan
        
        data["gps time - pc time"] = (gps_time_index-pc_time_index).total_seconds()
        
        
        if gps_type == "accessory":
            ##### tilt is the angle that the instrument is away from level
            data["tilt"] = np.sqrt(np.power(data["Roll"], 2)+np.power(data["Pitch"], 2))
        
        ##### plots the histograms
        dh = DailyHists(data.copy())
        try:
            dh.plot_one_hist("gps time - pc time", axes["time_diff"], ignore_zero=False, weight=True, ybuffer=False, log=True, linthresh=10, lin_ticks=3, log_ticks_skipped=0)#, ylims=[-10**10, 10**4])
        except Exception as e:
            print("failed plotting gps_time - pc_time comparison\n"+str(e))
        
        if gps_type == "accessory":
            try:
                dh.plot_one_hist("GPSAge", axes["gps_age"], log=True, ybuffer=True)
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed plotting GPSAge\n"+str(e))
            try:
                dh.plot_one_hist("NSV", axes["NSV"], limited_bins=True, zero_axes=True)
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed plotting NSV\n"+str(e))
            try:
                dh.plot_one_hist("tilt", axes["tilt"])
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed plotting tilt\n"+str(e))
        elif gps_type == "GPS":
            try:
                dh.plot_one_hist("baro_temp", axes["gps_age"])
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed plotting baro_temp\n"+str(e))
            try:
                dh.plot_one_hist("pressure", axes["NSV"])
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed plotting pressure\n"+str(e))
            try:
                dh.plot_one_hist("rh", axes["tilt"])
            except KeyboardInterrupt as e:
                raise e
            except Exception as e:
                print("failed plotting tilt\n"+str(e))
        
        lat_lon_name = ["Latitude", "Longitude"]
        if gps_type == "GPS":
            lat_lon_name = ["gps_latitude", "gps_longitude"]
            
            
        ##### plots the gps graphs
        ll = LatLonGraph(data)
        try:
            density_im = ll.plot_lat_lon(axes["gps_density"], stack_resolution="max", lat_lon_name=lat_lon_name)
            cbar = fig.colorbar(density_im, ax=axes["gps_density"], format='%.0f')
        except KeyboardInterrupt as e:
            raise e
        except Exception:
            print("failed plotting gps frequency")
        try:
            time_im = ll.plot_lat_lon(axes["gps_time"], "pc_time_end_measurement", stack_resolution="max", lat_lon_name=lat_lon_name)
            cbar = fig.colorbar(time_im, ax=axes["gps_time"])
            
            ##### generates the ticks and converts to string date format
            ticks = np.linspace(cbar.vmin, cbar.vmax, 5)
            labels = [datetime.fromtimestamp(int(tick/10**9)).strftime("%Y-%m-%d") for tick in ticks]
            cbar.ax.set_yticks(ticks, labels)
        except KeyboardInterrupt as e:
            raise e
        except Exception:
            print("failed plotting gps time")
        
        
        
        deployment_metadata = self.deployment_metadata
        if deployment_metadata is None:
            if self.driver is not None:
                deployment_metadata = self.driver.db_load.load_metadata()
        
        if title == "" and deployment_metadata is not None:
            title = deployment_metadata["deployment_description"][0]
        fig.suptitle(title+"\n")
        
        if deployment_metadata is not None:
            metadata_string = graphUtils.generate_metadata_string(deployment_metadata, data)
            plt.figtext(x=0.5, y=0.93, s=metadata_string, horizontalalignment="center")
    
        print("GPS plotted")
        if self.output_location is not None:
            plt.savefig(self.output_location+"/GPS_summary.png")
        plt.show()
    
    
    def __make_colormap(self):
        """makes a new colormap which is jet but a value of 0 is black rather than white
        returns: new colormap
        """
        black = np.array([0,0,0,1])
        
        jet = None
        ##### accounting for different matplotlib versions
        mpl_version = matplotlib.__version__
        mpl_version = mpl_version.split(".")
        if int(mpl_version[0]) > 3 or (int(mpl_version[0]) == 3 and int(mpl_version[1]) > 5):
            jet = matplotlib.colormaps["jet"].resampled(256)
        else:
            jet = matplotlib.colormaps["jet"]._resample(256)
            
        new_colors = jet(np.linspace(0, 1, 256))
        new_colors[0] = black
        new_cmap = matplotlib.colors.ListedColormap(new_colors)
        return new_cmap
    
    
    def load_spectrum(self, cutoff_angle, **kwargs):
        """loads the global spectrum from the database, taking into account cutoff angle
        params:
            cutoff_angle: None | float(radians) | tuple[float](radians)
                          the angle(s) range to include
        returns:
            spectrum: the requested spectrum, and time
        """
        spectrum = []
        if cutoff_angle is None:
            spectrum = self.driver.db_load.load(["pc_time_end_measurement", "global_spectrum"])
        elif type(cutoff_angle) == type((0, )):
            if len(cutoff_angle) == 2:
                condition="sza < "+str(cutoff_angle[1]) + " AND sza > "+str(cutoff_angle[0])
                spectrum = self.driver.db_load.load(["pc_time_end_measurement", "global_spectrum"], 
                                                    condition=condition, 
                                                    timezone=self.timezone, 
                                                    **kwargs)
            else:
                spectrum = self.driver.db_load.load(["pc_time_end_measurement", "global_spectrum"], 
                                                    timezone=self.timezone, 
                                                    **kwargs)
        else:
            spectrum = self.driver.db_load.load(["pc_time_end_measurement", "global_spectrum"], 
                                                timezone=self.timezone, 
                                                condition="sza < "+str(cutoff_angle), 
                                                **kwargs)
        
        if len(spectrum) == 0:
            raise Exception("could not load spectrum")
        
        return spectrum




# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from hsr1.plots import graphUtils
import hsr1.utils.spectrum.spectrumUtils as SpectrumUtils




class LinearDipsGraph:
    def plot_n_biggest_dips(self, global_spectrum, 
                            n=15,
                            cutoff_wavelength=1000,
                            date_format:str="%y-%m-%d %H:%M",
                            title=None):
        """one plot per day of the the n biggest peak for each measurement
        params:
            global_spectrum: dataframe containing spectral data
            n: the number of peaks that will be selected per measurement
            cutoff_wavelength: the maximum wavelength included
        """
        
        dates = global_spectrum["pc_time_end_measurement"].dt.tz_localize(None)
        
        
        global_spectrum = global_spectrum["global_spectrum"].values
        
        nlargest, nlargest_width, nlargest_prominence = SpectrumUtils.find_nlargest(global_spectrum, n, ["wavelengths", "widths", "prominences"], cutoff_wavelength)        
        
        x_axis = [[i]*n for i in dates]
        x_axis = np.array(sum(x_axis, []))
        x_axis = pd.to_datetime(x_axis).astype("int64")
        xlims = (x_axis[0], x_axis[-1])
        
        ##### removing nan data
        x_axis = x_axis[~np.isnan(nlargest)]
        nlargest_width = nlargest_width[~np.isnan(nlargest)]
        nlargest_prominence = nlargest_prominence[~np.isnan(nlargest)]
        nlargest = nlargest[~np.isnan(nlargest)]
        
        ##### generate list of widths, to plot point size
        max_width = np.nanmax(nlargest_width)
        nlargest_width = ((nlargest_width/max_width)*10)**2
        
        fig, axes = plt.subplots(1, figsize=(16.5, 11.7))
        
        if title is None:
            title = str(dates.iloc[0].date())+"  -  " + str(dates.iloc[len(dates)-1].date())
        fig.suptitle("prominent dip wavelengths\n" + str(title))
        
        scatter = axes.scatter(x_axis, nlargest, s=nlargest_width, c=nlargest_prominence, cmap="jet")
        
        x_min = x_axis[0]
        x_max = x_axis[-1]
        
        x_ticks = np.linspace(x_min, x_max, 10)
        x_labels = [timestamp.strftime(date_format) for timestamp in pd.to_datetime(x_ticks)]
        
        axes.set_xticks(x_ticks, x_labels)
        
        
        # x_axis_timestamp = pd.to_datetime(x_axis.to_numpy())
        # xticks = x_axis_timestamp[x_axis_timestamp.minute == 0].unique()
        
        
        # xlabels = [label.hour for label in xticks]
        
        # axes.set_xticks(xticks.astype("int64"), xlabels)
        
        axes.set_xlim(xlims)
        axes.set_ylim((300, cutoff_wavelength))
        
        graphUtils.plot_reference_lines_and_labels(axes, xlims[0])
        
        plt.show()        
        
    
    
    def plot_biggest_dips_day(self, global_spectrum, n=15, cutoff_wavelength=1000,
                               date_format:str="%H:%M"):
        global_spectrum["pc_time_end_measurement"] = pd.DatetimeIndex(global_spectrum["pc_time_end_measurement"].dt.tz_localize(None))
        
        days = global_spectrum["pc_time_end_measurement"].dt.date.unique()
        global_spectrum_days = [global_spectrum.loc[global_spectrum["pc_time_end_measurement"].dt.date == day] for day in days]
        for i, global_spectrum_day in enumerate(global_spectrum_days):
            self.plot_n_biggest_dips(global_spectrum_day, n, cutoff_wavelength, date_format="%H:%M", title=days[i])
    


# -*- coding: utf-8 -*-
"""
Created on Wed Feb 28 17:02:09 2024

@author: albie
"""


import hsr1.read_txt.ImportHSRFiles as ImportHSRFiles
import hsr1.read_txt.loadFromTxt as loadFromTxt
from hsr1.read_txt.loadFromTxt import *


# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""
import math

import matplotlib
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

import hsr1.plots.graphUtils as graphUtils
import hsr1.plots.flagData as flagData

class DailyPlots:
    def __init__(self, columns, data, title_prefix, output_location=None, flag=False, max_limit=None, min_limit=None):
        self.columns = columns
        self.title_prefix = title_prefix
        self.output_location = output_location
        self.flag = flag
        
        ##### if dni is one of the columns, set ylimit according to the next 100 from ghi
        ##### as dfi has big spikes near sunset
        min_max_columns = self.columns.copy()
        if "direct_normal_integral" in min_max_columns:
            min_max_columns.remove("direct_normal_integral")
            if not "global_integral" in min_max_columns:
                min_max_columns.append("global_integral")
                data["global_integral"] = ((((data["global_integral"]+10)/100)+1)*100).astype(int)
            
        self.max = np.nanmax(data[min_max_columns].replace({np.inf: np.nan}).values) if max_limit is None else max_limit
        self.min = np.nanmin(data[min_max_columns].replace({np.inf: np.nan}).values) if min_limit is None else min_limit
        
        self.flags = None
        self.flag_columns = None
        
        if flag:
            self.flags = flagData.flag(data, True)
            self.flag_columns = self.flags.columns
    
    
    def plot_row(self, axes, data, dates, day_format:str="%d", legend=False):
        df = data.copy()
        
        df["pc_time_end_measurement"] = pd.to_datetime(df["pc_time_end_measurement"])
        axes.set_xlim(min(df["pc_time_end_measurement"]), max(df["pc_time_end_measurement"]))
        for column in self.columns:
            label_column = column
            if label_column[0] == "_":
                label_column = label_column[1:]
            axes.plot(df["pc_time_end_measurement"], df[column], linewidth=0.5, label=label_column)
        
        if self.flag:
            df[self.flag_columns] = df[self.flag_columns].astype(float).fillna(0)
            any_flags = data[self.flag_columns].any(axis=1)
            df.loc[:, self.columns] = df.loc[:, self.columns].fillna(0)
            for column in self.columns:
                flag_df = pd.DataFrame()
                flag_df["pc_time_end_measurement"] = df["pc_time_end_measurement"]
                flag_df[column] =  df.loc[any_flags, column]
                axes.plot(flag_df["pc_time_end_measurement"], flag_df[column], color=(1,0,1), linewidth=0.5)
        
        axes.set_ylim(self.min, self.max)
        if legend:
            ##### accounting for different matplotlib versions
            mpl_version = matplotlib.__version__
            mpl_version = mpl_version.split(".")
            if int(mpl_version[0]) > 3 or (int(mpl_version[0]) == 3 and int(mpl_version[1]) > 5):
                legend = axes.legend(loc='upper center', ncols=len(self.columns))
                for leg in legend.legend_handles:
                    leg.set_linewidth(3)
            else:
                legend = axes.legend(loc='upper center', ncol=len(self.columns))
        
        labels = []
        for date in dates:
            labels.append(pd.Timestamp(date).strftime(day_format).lstrip("0")) if date != "" else labels.append("")
            labels.append("")
        labels.append("")
        
        xlims = axes.set_xlim()
        daynos = np.linspace(xlims[0], xlims[1], 2*(len(dates))+1)
        axes.set_xticks(daynos, labels)
    
    
    def plot_month(self, df, rows, days_in_row):
        if rows == None and days_in_row == None:
            rows = 5
            days_in_row = 7
        
        if rows == None:
            x = 31//days_in_row
            rows = x if 31 % days_in_row == 0 else x + 1
            
        elif days_in_row == None:
            x = 31//rows
            days_in_row = x if 31 % rows == 0 else x + 1
        
        month = str(df["pc_time_end_measurement"].iloc[0].year) + "-" + str(df["pc_time_end_measurement"].iloc[0].month)
        month_timestamp = pd.Timestamp(month)
        
        month_int = df["pc_time_end_measurement"].iloc[0].year*10+df["pc_time_end_measurement"].iloc[0].month
        
        row_dates = []
        for i in range(rows):
            row_dates.append([])
            for j in range(i*days_in_row, (i+1)*days_in_row):
                # row_dates[i].append((month_timestamp + pd.Timedelta(j, "days")).strftime("%Y-%m-%d"))
                row_dates[i].append((month_timestamp + pd.Timedelta(j, "days")).date())
        timestamps_in_month = df["pc_time_end_measurement"].dt.year*10+df["pc_time_end_measurement"].dt.month == month_int
        last_day_of_month = df.loc[timestamps_in_month].iloc[len(df)-1]["pc_time_end_measurement"].date
        day_one = pd.Timestamp(df['pc_time_end_measurement'].iloc[0])
        title = f"{day_one.strftime('%B')} {day_one.strftime('%Y')}"
        self.plot_page(row_dates, df, last_day_of_month, title)
    
    def plot_int(self, df:pd.DataFrame, period:int, rows:int=None, days_in_row:int=None):
        ##### if insufficient rows/days_in_row are given make a best guess from the period and any given values
        if rows == None and days_in_row == None:
            x = math.sqrt(period)
            rows = days_in_row = int(x) if x % 1 == 0 else int(x) + 1
            
        elif rows == None:
            x = period//days_in_row
            rows = x if period % days_in_row == 0 else x + 1
            
        elif days_in_row == None:
            x = period//rows
            days_in_row = x if period % rows == 0 else x + 1
        
        num_days_in_page = rows*days_in_row
        all_days = df["pc_time_end_measurement"].dt.date.unique()
        num_pages = len(all_days)//num_days_in_page if len(all_days) % num_days_in_page == 0 else len(all_days)//num_days_in_page + 1
        
        
        
        page_days = [all_days[i*num_days_in_page:(i+1)*num_days_in_page] for i in range(num_pages)]
        
        for days in page_days:
            
            first_day = days[0]
            row_dates = []
            for i in range(rows):
                this_row = []
                for j in range(days_in_row):
                    this_row.append(first_day + pd.Timedelta(i*days_in_row+j, "days"))
                row_dates.append(this_row)
            
            last_valid_day = all_days[-1]
            # if len(row_dates[-1]) < days_in_row:
            #     for i in range(days_in_row - len(row_dates[-1])):
            #         row_dates[-1] = np.append(row_dates[-1], (pd.Timestamp(last_valid_day) + pd.Timedelta(i+1, "days")).strftime("%Y-%m-%d"))
            
            title = ""
            months = pd.Series(pd.to_datetime(days)).dt.strftime("%B").unique()
            if len(months) == 1:
                title += months[0]
            else:
                title += ", ".join(months[:-1]) + " and " + months[-1]
            
            title += " "            
            years = pd.Series(pd.to_datetime(days)).dt.year.unique()
            if len(years) == 1:
                title += str(years[0])
            else:
                title += ", ".join(str(years[:-2])) + " and " + str(years[-1])
            
            
            self.plot_page(row_dates, df, last_valid_day, title, "%b-%d")
            
    
    
    def plot_page(self, row_dates, data, last_valid_day, title, day_format:str="%d"):
        df = data.copy()
        
        ##### dosent include last day, stops at the start of it
        all_seconds = [pd.date_range(pd.Timestamp(row[0]), pd.Timestamp(row[-1]) + pd.Timedelta(1, "day"), freq="min") for row in row_dates]
        
        all_seconds_dfs = [pd.DataFrame(row, columns=["pc_time_end_measurement"]) for row in all_seconds]
        
        df["pc_time_end_measurement"] = df["pc_time_end_measurement"].dt.tz_localize(None)
                
        row_dfs = [df.loc[np.isin(df["pc_time_end_measurement"].dt.date, row)] for row in row_dates]
        row_dfs = [pd.merge(all_seconds_dfs[i], row_dfs[i], on="pc_time_end_measurement", how="outer") for i in range(len(all_seconds_dfs))]
        
        full_title = self.title_prefix+title
        fig, axes = plt.subplots(len(row_dates), figsize=(16.5, 11.7))
        if isinstance(axes, matplotlib.axes._axes.Axes):
            axes = [axes]
        
        fig.suptitle(full_title)
        plt.tight_layout()
        
        last_valid_day_count = 0
        for i, row in enumerate(row_dates):
            for j, date in enumerate(row):
                if last_valid_day_count >= 1:
                    row_dates[i][j] = ""
                if date == last_valid_day:
                    last_valid_day_count += 1
                
        legend = True
        for i, df in enumerate(row_dfs):
            self.plot_row(axes[i], df, row_dates[i], day_format=day_format, legend=legend)
            legend=False
        
        full_title = graphUtils.make_full_title(full_title)
        
        if self.output_location is not None:
            plt.savefig(self.output_location+"/daily_plots_"+full_title+".png")
        plt.show()
    
    
    def plot_series(self, period, rows, days_in_row, data, ignore_zeros=False):
        """
        params:
            period: how many days per page, "monthly", "weekly", integer
        """
        
        # self.max_integral = (int((self.max_integral+10)/100)+1)*100
        if self.flag:
            data = pd.merge(data, self.flags, left_index=True, right_index=True)
        if ignore_zeros:
            data = data.replace({0:np.nan})
        
        pages_dfs = pd.DataFrame()
        
        if type(period) == type(0):
            self.plot_int(data, period, rows, days_in_row)
        
        elif period == "monthly":
            
            # years_and_months = np.unique(list(zip(data["pc_time_end_measurement"].dt.month.to_numpy(), 
            #                                       data["pc_time_end_measurement"].dt.year.to_numpy())), 
            #                              axis=0)
            years_and_months = data["pc_time_end_measurement"].dt.year*100+data["pc_time_end_measurement"].dt.month
            
            pages_dfs = [data.loc[data["pc_time_end_measurement"].dt.year*100+data["pc_time_end_measurement"].dt.month == year_and_month] for year_and_month in years_and_months.unique()]
        
            for df in pages_dfs:
                self.plot_month(df, rows, days_in_row)
        
        elif period == "weekly":
            if rows == None:
                rows = 5
            days_in_row = 7
            period = 7 * rows
            self.plot_int(data, period, rows, days_in_row)
        


# -*- coding: utf-8 -*-
"""
Created on Thu Feb 29 12:11:45 2024

@author: albie
"""

from hsr1.utils.reformatData import ReformatData
from hsr1.utils.config import Config
import hsr1.utils.HSRFunc as HsrFunc
import hsr1.utils.spectrum


# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""
import os

import hsr1

db_loc = "databases/synthetic/"

output_location = "C:/Users/albie/Dropbox/Albie (1)/synthetic dataset images"

winster = {"lat":53.14,
           "lon":-1.63,
           "alt":227.0}

n_pole = {"lat":90,
          "lon":0,
          "alt":0}

svalbard = {"lat":77,
            "lon":50,
            "alt":0}


def create_001(plot=True, **kwargs):
    """Standard: 
    spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 10s
    integral source: pvlib
    location: Winster
    duration: 1 month"""
    
    db_name = db_loc+"001.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_accessory_data("10s")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_001()
    
def plot_001(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "001"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_0011(plot=True, **kwargs):
    """Standard: 
    spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 10s
    integral source: pvlib
    location: Winster
    duration: 1 month"""
    
    db_name = db_loc+"0011.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2023-11-01 00:00:00+00:00",
                                                     end_date="2024-03-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("10min")
    dataset.generate_accessory_data("10min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_0011()
    
def plot_0011(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "0011"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)
    


def create_002(plot=True, **kwargs):
    """spectral_data period: 1min
    system_data period: 10min
    accessory_data period: 10s
    integral source: pvlib
    location: Winster
    Duration: 1 month"""
    
    db_name = db_loc+"002.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_system_data("10min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_002()

def plot_002(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "002"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_003(plot=True, **kwargs):
    """spectral_data period: 10min
    system_data period: 1h
    accessory_data period: 10min
    integral source: pvlib
    location: Winster
    Duration: 1 month"""
    
    db_name = db_loc+"003.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("10min")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_003()

def plot_003(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "003"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_004(plot=True, **kwargs):
    """spectral_data period: 1h
    system_data period: 1h
    accessory_data period: 1h
    integral source: pvlib
    location: Winster
    Duration: 1 month"""
    
    db_name = db_loc+"004.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_004()

def plot_004(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "004"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_005(plot=True, **kwargs):
    """spectral_data period: 1h
    system_data period: 1h
    accessory_data period: 1h
    integral source: pvlib
    location: Winster
    Duration: 1 month"""
    
    db_name = db_loc+"005.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2025-01-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_005()

def plot_005(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "005"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_006(plot=True, **kwargs):
    """spectral_data period: 10s
    system_data period: 1min
    accessory_data period: 10s
    Integral source: pvlib
    Location: Winster
    Duration: 1 Week
    """
    
    db_name = db_loc+"006.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-01-08 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("10s")
    dataset.generate_system_data("1min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_006()

def plot_006(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "006"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_007(plot=True, **kwargs):
    """spectral_data period: 10s
    system_data period: 1min
    accessory_data period: 10s
    Integral source: pvlib
    Location: Winster
    Duration: 1 Week
    Notes: data is not taken on the minute, instead 5s past the minute
    """
    
    db_name = db_loc+"007.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:05+00:00",
                                                     end_date="2024-01-08 23:59:05+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("10s")
    dataset.generate_system_data("1min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_007()

def plot_007(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "007"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_008(plot=True, **kwargs):
    """spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 10s
    Integral source: pvlib
    Location: Winster
    Duration: 1 day
    """
    
    db_name = db_loc+"008.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:05+00:00",
                                                     end_date="2024-01-01 23:59:05+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_system_data("1min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_008()

def plot_008(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "008"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_009(plot=True, **kwargs):
    """spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 10s
    Integral source: pvlib
    Location: Winster
    Duration: 1 day
    note: missing data at 12 midday
    """
    
    db_name = db_loc+"009.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:05+00:00",
                                                     end_date="2024-01-01 23:59:05+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_system_data("1min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    for i in range(12):
        dataset.remove_rows("spectral_data", dataset.remove_n_hour, i)
    dataset.remove_rows("spectral_data", dataset.remove_n_hour, 23)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_009()

def plot_009(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "009"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_010(plot=True, **kwargs):
    """spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 10s
    Integral source: pvlib
    Location: north pole
    Duration: 1 Week
    """
    
    db_name = db_loc+"010.db"
    
    location = n_pole
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-06-01 00:00:00+00:00",
                                                     end_date="2024-06-08 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_system_data("1min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_010()

def plot_010(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "010"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_011(plot=True, **kwargs):
    """spectral_data period: 1h
    system_data period: 1h
    accessory_data period: 1h
    Integral source: pvlib
    Location: north pole
    Duration: 1 Year
    """
    
    db_name = db_loc+"011.db"
    
    location = n_pole
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2025-01-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_011()

def plot_011(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "011"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_012(plot=True, **kwargs):
    """spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 1min
    Integral source: pvlib
    Location: north pole
    Duration: 2 months(over the sunrise day)
    """
    
    db_name = db_loc+"012.db"
    
    location = n_pole
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-03-01 00:00:00+00:00",
                                                     end_date="2024-05-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_012()

def plot_012(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "012"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_013(plot=True, **kwargs):
    """spectral_data period: 1h
    system_data period: 1h
    accessory_data period: 1h
    Integral source: pvlib
    Location: svalbard
    Duration: 1 Year
    """
    
    db_name = db_loc+"013.db"
    
    location = svalbard
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2025-01-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_013()

def plot_013(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "013"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_014(plot=True, **kwargs):
    """spectral_data period: 1h
    system_data period: 1h
    accessory_data period: 1h
    Integral source: pvlib
    Location: svalbard
    Duration: 4 Months
    """
    
    db_name = db_loc+"014.db"
    
    location = svalbard
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-05-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_014()

def plot_014(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "014"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_100(plot=True, **kwargs):
    """spectral_data period: 1min
    system_data period: 1min
    accessory_data period: 1min
    Integral source: hour of day
    Location: winster
    Duration: 1 Month
    """
    
    db_name = db_loc+"100.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_system_data("1min")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.integral_hour_of_day)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_100(**kwargs)

def plot_100(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "100"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_101(plot=True, **kwargs):
    """spectral_data period: 1h
    system_data period: 1h
    accessory_data period: 1h
    Integral source: hour of day
    Location: winster
    Duration: 1 Month
    """
    
    db_name = db_loc+"101.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1h")
    dataset.generate_system_data("1h")
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.integral_hour_of_day)
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_101(**kwargs)

def plot_101(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "101"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_102(plot=True, **kwargs):
    """spectral_data period: 1m
    system_data period: 1h
    accessory_data period: 10s
    Integral source: pvlib
    Location: winster
    Duration: 1 Month
    system data calculated from accesory_data
    data is mobile, no default latitude/longitude, should have gaps in elv/azi plot
        where precalculated_values couldnt be calculated
    """
    
    db_name = db_loc+"102.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 00:00:00+00:00",
                                                     end_date="2024-02-01 23:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.generate_accessory_data("10s")
    dataset.remove_rows("accessory_data", dataset.remove_n_hour, 12)
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    def return_speech_marks():
        return ["''"]*3
    
    dataset.generate_custom_column("deployment_metadata", ["default_elevation", "default_latitude", "default_longitude"], return_speech_marks)
    
    
    dfs = dataset.get_dfs()
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_102(**kwargs)

def plot_102(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "102"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)


def create_103(plot=True, **kwargs):
    """spectral_data period: 1m
    accessory_data period: 10s
    Integral source: pvlib
    Location: winster
    Duration: 1 Day
    system data calculated from accesory_data
    missing data from 12-3pm
    """
    
    db_name = db_loc+"103.db"
    
    location = winster
    
    if os.path.exists(db_name):
        print("overwriting database")
        os.remove(db_name)

    dataset = hsr1.synthetic_dataset.SyntheticDataset(start_date="2024-01-01 09:00:00+00:00",
                                                     end_date="2024-01-01 19:59:00+00:00",
                                                     latitude=location["lat"],
                                                     longitude=location["lon"],
                                                     altitude=location["alt"])
    dataset.generate_spectral_data("1min")
    dataset.remove_rows("spectral_data", dataset.remove_n_hour, 12)
    dataset.remove_rows("spectral_data", dataset.remove_n_hour, 13)
    dataset.remove_rows("spectral_data", dataset.remove_n_hour, 14)
    
    dataset.generate_accessory_data("10s")
    
    dataset.generate_custom_column("accessory_data", "Latitude", dataset.num_readings, base=location["lat"], scale=0.001)
    dataset.generate_custom_column("accessory_data", "Longitude", dataset.num_readings, base=location["lon"], scale=0.001)
    
    dataset.generate_system_data_accessory()
    dataset.generate_deployment_metadata("C:/Users/albie/work/Datasets/Winster 2023/HSR1-002 Winster 2024 Deployment.ini")
    
    dataset.generate_custom_column("spectral_data", 
                                   ["global_integral", "diffuse_integral"], 
                                   dataset.pvlib_integral_static)
    
    # def return_speech_marks():
    #     return ["''"]*3
    
    # dataset.generate_custom_column("deployment_metadata", ["default_elevation", "default_latitude", "default_longitude"], return_speech_marks)
    
    
    dfs = dataset.get_dfs()
    dfs = (dfs[0], dfs[1], dfs[2])
    
    db = hsr1.DBDriver(db_name)
    db.store(dfs)
    
    if plot:
        plot_103(**kwargs)

def plot_103(integral=True, gps=False, dips=False, accessory=False, plot_all=False, **kwargs):
    db_num = "103"
    
    if plot_all:
        integral=True
        gps=True
        dips=True
        accessory=True
    
    db_name = db_loc+db_num+".db"
    graph = hsr1.Graph(db_name, output_location=output_location+"/"+db_num, **kwargs)
    
    if integral:
        graph.plot_integral(title="synthetic dataset "+db_num, **kwargs)
    if gps:
        graph.plot_gps(title="synthetic dataset "+db_num)
    if dips:
        graph.plot_dips_summary(title="synthetic dataset "+db_num)
    if accessory:
        graph.plot_accessory(title="synthetic dataset "+db_num)





# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""

import pandas as pd
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap

from hsr1.plots import graphUtils

import time

class TimeDayGraph:
    def __init__(self, cmap=plt.get_cmap("jet"), timedelta=pd.Timedelta(0), diffuse_name="dif"):
        self.diffuse_name = diffuse_name
        self.cmap = cmap
        ##### according to brsn limits, highest possible global value is 1407
        self.max_integral = None
        self.requirements = {"all":["pc_time_end_measurement", "global_integral", "diffuse_integral", "direct_normal_integral", "gps_longitude", "gps_latitude", "gps_altitude"],
                             "ghi":["pc_time_end_measurement", "global_integral", "gps_longitude", "gps_latitude", "gps_altitude"],
                             "diff":["pc_time_end_measurement", "diffuse_integral", "gps_longitude", "gps_latitude", "gps_altitude"],
                             "dni":["pc_time_end_measurement", "direct_normal_integral", "gps_longitude", "gps_latitude", "gps_altitude"]}
        self.flags = None
        self.timedelta = timedelta
        
    
    def time_day_graph(self, 
                       df:pd.DataFrame, 
                       axes, 
                       time_col:str, 
                       data_col:str, 
                       sunrise_sunset=None, 
                       show_xlabels:bool=True, 
                       show_cbar=True,
                       stack_resolution="max"):
        """plots one column of data onto a plot with minutes in a day in one axes and days on another
        params:
            df: data to be plotted
            axes: axes on which data will be plotted
            time_col: name of column in data that contains the time of each datapoint
            data_col: name of column in data that contains the data to be plotted
            sunrise_sunset: the sunrise and sunset data. if None, not plotted
            show_xlabels: whether or not to show the x axes labels. useful when plotting multiple of these graphs with the same labels
            show_cbar: whether or not to show the colorbar. useful when plotting multiple of these graphs with the same colormap
            stack_resolution: ("max"/"mean"/"min") when there are more readings
                than pixels, how to resolve the multiple readings that correspond
                to each pixel.
        """
        if self.max_integral is None:
            self.max_integral = np.nanmax(df[data_col])
        
            
        bbox = axes.get_window_extent()
        vert_pixels = int(bbox.height)
        
        datetimeindex = pd.DatetimeIndex(df[time_col]).tz_localize(None)
        
        start = df.loc[0, time_col].tz_localize(None)
        stop = df.loc[len(df)-1, time_col].tz_localize(None)
        dates = pd.date_range(start, stop, normalize=True).date
        
        ##### generates the new index seperated by days and minutes
        new_index = [datetimeindex.date, datetimeindex.hour*60+datetimeindex.minute+datetimeindex.second/60]
        
        ##### sets the new index and unstacks it, which converts the dataframe from having 2 indexes,
        #####   to having 1 index, and for each column, having n columns, where n is the number of days
        new_df = df.set_index(new_index).unstack(level=0)
        
        ##### convert to hours, more human readable
        new_df.index = new_df.index/60
        
        new_df = new_df[data_col]
        new_df = new_df.reindex(columns=dates)
        
        extent = (pd.to_datetime(dates[0]).timestamp(), pd.to_datetime(dates[-1]+pd.Timedelta(1, "day")).timestamp(), 24, 0)
        
        ##### find the smallest number of groups that can represent all 
        #####   the data loslessly. if impossible use the max number of groups
        #####   that will fit onto the graph
        diff = np.diff(new_df.index)
        min_diff = np.min(diff[diff>0])
        
        ##### convert to hours
        num_groups = round(24/min_diff)
        pixels_to_use = min(num_groups, vert_pixels)

        pixel_hours = np.linspace(0, 24, pixels_to_use, endpoint=False)
        measured_hours = np.array(new_df.index)
        
        # ##### finds the closest pixel to each reading's true value
        def closest_pixel_hour(i):
            return pixel_hours[np.argmin(np.abs(pixel_hours-i))]
        
        ##### groups all the readings into one pixel groups, according to the
        #####   stack_resolution parameter
        if stack_resolution == "max":
            new_df = new_df.groupby(closest_pixel_hour).max()
        elif stack_resolution == "mean":
            new_df = new_df.groupby(closest_pixel_hour).mean()
        elif stack_resolution == "min":
            new_df = new_df.groupby(closest_pixel_hour).min()
        
        ##### fill the df with nan where there are no readings
        all_pixel_hours = pd.DataFrame(index=pixel_hours)
        new_df = pd.merge(new_df, all_pixel_hours, how="outer", left_index=True, right_index=True)
        
        image = axes.imshow(new_df, aspect="auto", cmap=self.cmap, interpolation="none", extent=extent, vmin=0, vmax=self.max_integral)
        axes.set_ylim(24, 0)
        
        xlim = axes.get_xlim()
        
        
        ##### plots the sunrise and sunset lines
        if sunrise_sunset is not None:
            sunrise_set_date = np.linspace(xlim[0], xlim[1], len(sunrise_sunset))
            
            sunrise_time = sunrise_sunset["sunrise"].dt.hour + sunrise_sunset["sunrise"].dt.minute/60
            sunset_time = sunrise_sunset["sunset"].dt.hour + sunrise_sunset["sunset"].dt.minute/60
            
            if len(sunrise_set_date) == 1:
                sunrise_set_date = np.array([sunrise_set_date[0], sunrise_set_date[0] + 1])
                sunrise_time = np.array([sunrise_time]*2)
                sunset_time = np.array([sunset_time]*2)
            
            axes.plot(sunrise_set_date, sunrise_time, "--", color="white", linewidth=0.5)
            axes.plot(sunrise_set_date, sunset_time, "--", color="white", linewidth=0.5)
        
        
        ##### superimposes the passed flagged data onto the plot
        if self.flags is not None:
            colordict = {
                "red":((0,1,1), (1,1,1)),
                "green":((0,1,1), (1,0,0)),
                "blue":((0,1,1), (1,1,1))}
            flag_cmap = LinearSegmentedColormap("LinearCmap", colordict)
            
            ##### same process as the data graph but with the flagged data
            any_flags = pd.DataFrame()
            any_flags["flags"] = self.flags.any(axis=1)
            new_flag_df = any_flags.set_index(new_index).unstack(level=0)
            new_flag_df = new_flag_df["flags"]
            new_flag_df.index = new_flag_df.index/60
            new_flag_df = new_flag_df.reindex(columns=dates).astype(float)
            
            new_flag_df = new_flag_df.fillna(0)
            
            image = axes.imshow(new_flag_df, aspect="auto", norm=matplotlib.colors.LogNorm(vmin=0, vmax=1), cmap=flag_cmap, interpolation="none", extent=extent)
            
        
        xticks, labels = graphUtils.calculate_date_labels(df[time_col], axes.get_xlim())
        
        if not show_xlabels:
            labels = []
        
        axes.set_xticks(xticks, labels)
        axes.set_yticks(np.arange(0, 25, 6))
        
        if show_cbar:
            self.fig.colorbar(image, ax=axes, pad=0)
    
    
    def ghi(self, axes, df, sunrise_sunset=None, 
            show_xlabels:bool=True, show_cbar:bool=True):
        """graphs a time/day plot for ghi
        params:
            axes: the axes to plot on
            df: the data to be plotted
            sunrise_sunset: times of sunrise and sunset for every day
            show_xlabels: wether or not to show labels on the x axis, useful when plotting multiple at once
            show_cbar: wether or not to show a colorbar, useful when plotting multiple at once
        """
        self.time_day_graph(df, axes, "pc_time_end_measurement", "global_integral", 
                            sunrise_sunset=sunrise_sunset, 
                            show_xlabels=show_xlabels, show_cbar=show_cbar)
        axes.set_ylabel("GHI(W/m²)")
    
    def diff(self, axes, df, sunrise_sunset=None, 
             show_xlabels:bool=True, show_cbar:bool=True):
        """graphs a time/day plot for diffuse
        params:
            axes: the axes to plot on
            df: the data to be plotted
            sunrise_sunset: times of sunrise and sunset for every day
            show_xlabels: wether or not to show labels on the x axis, useful when plotting multiple at once
            show_cbar: wether or not to show a colorbar, useful when plotting multiple at once
        """
        self.time_day_graph(df, axes, "pc_time_end_measurement", "diffuse_integral", 
                            sunrise_sunset=sunrise_sunset, 
                            show_xlabels=show_xlabels, show_cbar=show_cbar)
        axes.set_ylabel(self.diffuse_name+"(W/m²)")
    
    def dni(self, axes, df, sunrise_sunset=None, 
            show_xlabels:bool=True, show_cbar:bool=True):
        """graphs a time/day plot for dni
        params:
            axes: the axes to plot on
            df: the data to be plotted
            sunrise_sunset: times of sunrise and sunset for every day
            show_xlabels: wether or not to show labels on the x axis, useful when plotting multiple at once
            show_cbar: wether or not to show a colorbar, useful when plotting multiple at once
        """
        self.time_day_graph(df, axes, "pc_time_end_measurement", "direct_normal_integral", 
                            sunrise_sunset=sunrise_sunset, 
                            show_xlabels=show_xlabels, show_cbar=show_cbar)
        axes.set_ylabel("DNI(W/m²)")

        
    
    def graph_all(self, axes, df, fig, show_cbar=True, flags=None, timedelta=None, plot_sunrise_sunset=True):
        """plots all the possible graphs of this type
        params:
            axes: list of axes that the graphs will be plotted on
            df: the data to be plotted
            fig: the figure all the graphs are plotted on, used for drawing the colormap
            show_cbar: wether or not to show a colorbar, useful when plotting multiple at once
            flags: dataframe of the flagged status of the data
            timedelta: pandas timedelta representing the timezone of the data
            plot_sunrise_sunset: wether or not to plot sunrise and sunset times
        """
        self.fig = fig
        self.flags = flags
        
        sunrise_set = None
        if plot_sunrise_sunset:
            sunrise_set = graphUtils.calculate_sunrise_sunset(df[["pc_time_end_measurement", "gps_longitude", "gps_latitude", "gps_altitude"]], timedelta)
        
        # TODO: check relevant columns are in the df
        self.ghi(axes[0], df[self.requirements["ghi"]], sunrise_sunset=sunrise_set, show_xlabels=False, show_cbar=False)
        self.diff(axes[1], df[self.requirements["diff"]], sunrise_sunset=sunrise_set, show_xlabels=False, show_cbar=False)
        self.dni(axes[2], df[self.requirements["dni"]], sunrise_sunset=sunrise_set, show_xlabels=True, show_cbar=False)
        
        if show_cbar:
            graphUtils.cbar(axes, fig, self.max_integral, 100)
        
        


# -*- coding: utf-8 -*-
"""
copyright 2024 Peak Design
this file is part of hsr1, which is distributed under the GNU Lesser General Public License v3 (LGPL)
"""
import os

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy import signal

def apply_polynomial(x, c0, c1, c2, c3):
    # return c3*x**3 + c2*x**2 + c1*x + c0
    return np.polyval([c3, c2, c1, c0], x)


def apply_inverse_polynomial(y, c0, c1, c2, c3,
                             target_point=600,
                             density=10,
                             sample_x_min=-1000, sample_y_max=2000):
    """for a given value y and a polynolmial, finds the value x that when transformed by the polynomial, gives y
    generates a sample of x and y data, and interpolates.
        but interpolation only works in a 1 to 1 graph (if each y point corresponds to multiple x points, which do you choose?)
        Most of this function is spent finding the segment of the graph that is one-one, and can be interpolated along
        by default it is the segment containg x=600, as that is in the middle of the spectrum, so will likley give the correct polynomial segment
    """
    sample_x = np.arange(sample_x_min, sample_y_max, 1/density)
    sample_y = np.polyval([c3, c2, c1, c0], sample_x)
    
    ##### finite differentiation(array of differences)
    grad = np.gradient(sample_y)
    
    ##### checks if polynomial is already 1-1
    ##### if gradient never crosses 0 -> is never flat -> never changes direction -> 1-1
    if (grad > 0).all() or (grad < 0).all():
        segment_x = sample_x
        segment_y = sample_y
    else:
        midpoint_index = None
        increasing_segment_x = None
        increasing_segment_grad = None
        ##### gradient formula = 3c3x^2 + ...
        #####  so if gradient c2 has the same sign as poly c3
        ##### if gradient c2 is pos, gradient is u-shaped, if negative, n-shaped
        if c3 >= 0:
            ##### gradient has minima
            midpoint_index = np.argmin(grad)
            
            ##### finds half of the gradient that is 1-1(either side of the minima/maxima is 1-1)
            increasing_segment_x = sample_x[midpoint_index:]
            increasing_segment_grad = grad[midpoint_index:]
        elif c3 < 0:
            ##### gradient has maxima
            midpoint_index = np.argmax(grad)
            
            ##### finds half of the gradient that is 1-1(either side of the minima/maxima is 1-1)
            increasing_segment_x = sample_x[:midpoint_index]
            increasing_segment_grad = grad[:midpoint_index]
        
        ##### finds the value where gradient is 0, poly is flat
        zero_pos = np.interp(0, increasing_segment_grad, increasing_segment_x)
        
        ##### x^2 is symetrical, find the other x-intercept, using the midpoint
        dist_from_mid = (zero_pos-sample_x[midpoint_index])
        zero_pos_2 = sample_x[midpoint_index] - dist_from_mid
        
        ##### convert values to index position
        zero_pos_index = int(midpoint_index + dist_from_mid*density)
        ##### index cant be less than 0 or more than length of array
        #####  this happens when only 1 turning point is within range
        zero_pos_2_index = min(len(sample_x), max(0, int(midpoint_index - dist_from_mid*density)))
        
        zero_pos = int(zero_pos)
        zero_pos_2 = int(zero_pos_2)
        
        
        ##### find the segment of the polynomial that is 1-1 and contains the target_pointx
        ##### if target point is to right of both turning points, use segment = rightmost turning point onwards
        if zero_pos <= target_point and zero_pos_2 <= target_point:
            segment_x = sample_x[max(zero_pos_index, zero_pos_2_index):]
            segment_y = sample_y[max(zero_pos_index, zero_pos_2_index):]
        ##### if target point is to left of both turning points, use segment = sample_x until leftmost turning point
        elif zero_pos >= target_point and zero_pos_2 >= target_point:
            segment_x = sample_x[min(zero_pos_index, zero_pos_2_index):]
            segment_y = sample_y[min(zero_pos_index, zero_pos_2_index):]
        ##### target point must be inbetween
        elif zero_pos <= zero_pos_2:
            segment_x = sample_x[zero_pos_index:zero_pos_2_index]
            segment_y = sample_y[zero_pos_index:zero_pos_2_index]
        elif zero_pos >= zero_pos_2:
            segment_x = sample_x[zero_pos_2_index:zero_pos_index]
            segment_y = sample_y[zero_pos_2_index:zero_pos_index]
    
    
    result = None
    if segment_y[0] > segment_y[-1]:
        ##### decreasing
        result = np.interp(y, np.flip(segment_y), np.flip(segment_x), left = 12345, right=-12345)
    else:
        ##### increasing
        result = np.interp(y, segment_y, segment_x, left = 12345, right=-12345)
    
    return result

def read_simple_file(filepath):
    """rands a simple file, one float per line"""
    data = []
    with open(filepath, "r") as f:
        data = f.readlines()

    for i in range(len(data)-1):
        data[i] = float(data[i][:-1])
    
    data[len(data)-1] = float(data[len(data)-1])

    data = np.array(data)
    
    return data

def read_reference_file(filepath):
    """reads a csv file containing a reference spectrum"""
    data = pd.read_csv(filepath, sep="\t")
    ordered = np.interp(np.arange(300, 1100), data["wavelength"], data["average"])
    return ordered

def read_smarts_file(filepath):
    raise NotImplementedError("change to read_simple_file()")

def read_amplitude_file(file_path):
    ##### read calibration data as df
    data = pd.read_csv(file_path, sep="\t", header=None)
    data.columns = ["wavelength_pos", "calibration"]
    
    ##### average out duplicate wavelength calibrations
    data = data.groupby("wavelength_pos").mean()
    data = data.reset_index()
    
    ##### convert to 1d numpy array
    calibration_array = data["calibration"].to_numpy()
    return calibration_array


def add_curves_to_graph(graph, curve, scale):
    plt.plot(graph)
    plt.show()
    plt.plot(curve)
    plt.show()
    
    
    curve_mid = int(len(curve)/2)
    
    max_value = max(graph)
    graph /= max_value
    
    for i in range(len(graph)):
        new_graph = np.zeros(len(graph))
        start = max(0, i-curve_mid)
        stop = min(len(graph), i+curve_mid)
        
        curve_start = max(0, curve_mid-i)
        curve_stop = min(len(curve), len(graph)-i)
        
        new_graph[start:stop] += curve[curve_start:curve_stop]
        plt.plot(new_graph)
        plt.show()

def poly_to_string(poly):
    return f"{poly[3]:.2e}x^3 + {poly[2]:.2e}x^2 + {poly[1]:.2f}x + {poly[0]:.2f}"


def find_nlargest(spectra, n, properties_to_find:[str], cutoff_wavelength, offset=300, find_peaks=False, fill_nan=False):
    """given a spectrum, find the n largest dips for each measurement
    params:
        spectra: dataframe containing one spectrum per row
        n: how many dips to pick per measurement
        properties_to_find: list of all properties to find, should always include prominences and wavelengths
        cutoff_wavelength: the maximum wavelength included
    """
    
    if type(cutoff_wavelength) in [int, float]:
        cutoff_wavelength = (offset, cutoff_wavelength)
        
    ##### generate base output array
    output_array = []
    for i in range(len(properties_to_find)):
        output_array.append([])
        
    ##### for each spectrum
    for spectrum in spectra:
        
        ##### cutoff wavelength
        spectrum = spectrum[(cutoff_wavelength[0]-offset):(cutoff_wavelength[1]-offset)]
        
        if not find_peaks:
            spectrum = -spectrum
        
        ##### find dips
        nn = (None, None)
        dips, properties = signal.find_peaks(spectrum, prominence=nn, width=nn)
        
        if n==0:
            new_n = len(dips)
        else:
            new_n = n
        
        if type(spectrum) == type(np.nan):
            for j in range(len(output_array)):
                output_array[j] += [np.nan]*new_n
            continue
        
        
        ##### convert from index to wavelength
        properties["wavelengths"] = dips+offset
        ##### add all the properties to a list
        data_list = []
        
        
        for _property in properties_to_find:
            data_list.append(properties[_property])
        
        ##### find the correct order of data, sorted by prominence
        #####   argsort so the same sort can be used on the other rows
        indexes = properties["prominences"].argsort()
        
        
        for j in range(len(properties_to_find)):
            ##### apply the sort and take the n most prominent
            data_list[j] = np.take_along_axis(np.array(data_list[j]), indexes, 0)[-new_n:]
            num_dips = len(data_list[j])
            if num_dips != n:
                data_list[j] = list(data_list[j])+[np.nan]*(n-num_dips)
            output_array[j] += list(data_list[j])
        
    
    for i, array in enumerate(output_array):
        output_array[i] = np.array(array)
    return output_array


def read_wavelength_polys(file_path):
    with open(file_path, "r") as f:
        lines = f.readlines()
    
    for i in range(len(lines)):
        lines[i] = lines[i].split(sep="\t")
    
    wavelength_polys = {}
    for line in lines:
        if line[0] == "WavelengthPoly":
            values = line[2:6]
            for i in range(len(values)):
                values[i] = float(values[i])
            wavelength_polys["channel_"+str(line[1])] = values
    
    return wavelength_polys

def format_output(cals, filepath):
    if os.path.dirname(filepath) != '':
        os.makedirs(os.path.dirname(filepath), exist_ok=True)
    
    try:
        with open(filepath, "w") as f:
            cals["average"] = np.array([0.0, 0.0, 0.0, 0.0])
            for channel in list(cals.keys())[:-1]:
                data = cals[channel]
                f.write("WavelengthPoly\t"+str(channel)[-1] +"\t"+ str(data[0]).replace("e", "E")+"\t"+str(data[1]).replace("e", "E")+"\t"+str(data[2]).replace("e", "E")+"\t"+str(data[3]).replace("e", "E")+"\n")
                
                ##### add current row to average
                if channel[-2] == "_" and int(channel[-1]) in np.arange(1, 8):
                    cals["average"] += np.array(data)
            
            cals["average"] = cals["average"]/7
            data = cals["average"]
            f.write("\nWavelengthPoly\tav(1-7)\t"+ str(data[0]).replace("e", "E")+"\t"+str(data[1]).replace("e", "E")+"\t"+str(data[2]).replace("e", "E")+"\t"+str(data[3]).replace("e", "E")+"\n")
                    
    except PermissionError as e:
        raise PermissionError("Permission to store results was denied, this is usually caused by trying to store directly to the root folder \n"+
                              str(e))


def apply_inverse_polynomial_a(y, c0, c1, c2, c3, flip=True):
    sample_x = np.arange(-2400, 2401)
    sample_y = apply_polynomial(sample_x, c0, c1, c2, c3)
    plt.plot(sample_x, sample_y)
    plt.axhline(y)
    
    sample_x = np.flip(sample_x)
    if flip:
        sample_y = np.flip(sample_y)
    
    corresponding_pixels = np.interp(y, sample_y, sample_x, right=12345, left = -12345)
    plt.axvline(corresponding_pixels)
    plt.show()
    return corresponding_pixels
